package org.warp.commonutils.concurrency.executor;

import java.util.HashSet;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.Executor;
import org.jetbrains.annotations.NotNull;

public abstract class ExecutorDecorator implements Executor {
	private final Executor executor;

	public ExecutorDecorator(Executor executor) {
		this.executor = Objects.requireNonNull(executor);
	}

	public final Set<Class<? extends ExecutorDecorator>> getExecutorDecorators() {
		if (executor instanceof ExecutorDecorator) {
			var decorators = ((ExecutorDecorator) executor).getExecutorDecorators();
			decorators.add(this.getClass());
			return decorators;
		} else {
			return new HashSet<>();
		}
	}

	@Override
	public void execute(@NotNull Runnable runnable) {
		executor.execute(runnable);
	}
}
